#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds
import hmScripts.hm_global_variables as gVar


def smartCombine(sel):
	'''
	Combine the provided inputs in one mesh while keeping
	the original meshes visible.
	'''
	# sel = cmds.ls(sl=True, l=True)
	combined = cmds.createNode('mesh')
	combinedTransform = cmds.listRelatives(combined, p=True, type='transform')[0]
	combineNode = cmds.createNode('polyUnite')
	
	cmds.connectAttr( combineNode + '.output', combined + '.inMesh')
	
	
	for x,el in enumerate(sel):
		cmds.connectAttr( el + '.outMesh', combineNode + '.inputPoly[%s]' % x)
		cmds.connectAttr( el + '.worldMatrix[0]', combineNode + '.inputMat[%s]' % x )
		
	return [combinedTransform, combined]


def removeCustomAttr(inputMesh):
	'''
	This function removes all the custom attributes foud on the passed mesh transform.
	If out connections are found from hmDivisions either polySmooth and smoothedShape are deleted.
	'''
	obj = inputMesh
	divisionAttr = gVar.smooth_attr_name
	
	#Define a list with all the attributes to delete
	customAttrList = [gVar.smooth_attr_name, gVar.offset_attr_name, gVar.operation_type,
					  gVar.outMeshToCompound, gVar.inMeshForCompoundA, gVar.inMeshForCompoundB,
					  gVar.outCompoundToMesh, gVar.inMeshFromCompound
					  ]
	
	#Get all the custom attributes
	custAttributes = cmds.listAttr(obj, ud=True)

	# === If the list isn't empty...
	if custAttributes != None:
		# for custAttr in custAttributes:
			# #print custAttr
					
			# # === Delete out smooth and shape
			# if custAttr == divisionAttr:
				# #Get the smooth node
				# currentSmooth = cmds.listConnections('%s.%s' % (obj, custAttr), d=True, type = 'polySmoothFace')
				# #print currentSmooth
				# #if smooth node
				# if currentSmooth != None:
					# #Get the out shape
					# outShape = cmds.listConnections('%s.output' % currentSmooth[0], d=True, shapes = True, type = 'mesh')
					# #if out shape
					# if outShape != None:
						# #Delete shape
						# print outShape[0]
						# cmds.delete(outShape[0])
					# else:
						# #if no shapes, just delete the smooth
						# #Delete smooth
						# cmds.delete(currentSmooth[0])
						
		
		# === Delete all custom Attr
		for custAttr in custAttributes:
			if custAttr in customAttrList:
				cmds.deleteAttr('%s.%s' % (obj, custAttr) )

				
	def checkIfCameFromNetwork(self, inputMesh ):
		'''
		It's used on the resulting meshes of others networks.
		We lock the smooth, remove from the original mesh layer and create useful connections for the smartEdit.
		In this way we will avoid to smooth mesh coming from an hm network in an automatic way, and we keep track 
		of the connection cascade creating a new connection with the current compound.
		'''

		
		meshType = queryMeshType( inputMesh )
		
		# print 'meshType is: ', meshType
		
		if meshType[0] == True:
			#If the mesh type is "in".. means that its the out of an existing hmCompound
			if meshType[1] == 'in' or meshType[1] == 'multi':
				#This just means that we found an hm attribute.. doesn't mean nothing more.
				#Check if we have connections....
				connections = cmds.listConnections('%s.%s' % (inputMesh, gVar.inMeshFromCompound ), d=True)
				
				#If the attribute is connected, we go further in dept with the searching
				if connections != None:	
					#We are assuming that the mesh is connected with another hmCompound as input...
					
					#Remove from the layer adding
					# self.originalMeshForLayer.remove(inputMesh)

					#Here we get the connected compound
					connectedCompound = connections[0]		
					
					#Connect the found compound to the current one
					try:
						cmds.connectAttr('%s.%s' % (connectedCompound, gVar.outCompoundToMesh), '%s.%s' % (self.compoundNode, gVar.outCompoundToMesh))
					except:
						cmds.warning('Something went wrong during the connection process...')
						
					#Display a warning message
					# cmds.warning('%s mesh is the result of another Hard Mesh Operation, forcing smooth to 0.' % inputMesh )
					
					#The value of the return
					return True
					
				#If our attribute is not connected, just return it as false, since means that is no more connected...
				else:
					# print 'nothing connected'
					return False
		else:
			# print 'not from network'
			return False				


def makeItWireframe(inputMesh, operation = 1 ):
	'''
	we activate the shading override to see through the meshes...
	'''
	shape = cmds.listRelatives( inputMesh, fullPath=True, shapes = True ) [0]
	try:
		#Activate the override
		if operation == 1:
			cmds.setAttr('%s.overrideEnabled' % shape, 1)
			cmds.setAttr('%s.overrideShading' % shape, 0)
		#De-Activate the override
		if operation == 0:
			cmds.setAttr('%s.overrideEnabled' % shape, 0)
			cmds.setAttr('%s.overrideShading' % shape, 1)
	except:
		cmds.warning('Hm Warning, unable remove shading from mesh: %s, maybe there are some other connections?' % shape)
		#Questo a quanto pare nasconde le mesh che sono risultato di un network nel creation..
		cmds.setAttr('%s.visibility' % inputMesh, 0)
			
			
			
# OBSOLETEEEEE!!!!!		
def queryMeshType(inputMesh):
	'''
	This function queryes the provided mesh to see if it's part of an hmNetwork
	or not. If yes we can know if it's a inputMesh for the compound or an outputMesh
	We also provide the attribute matches so we can use it for connection queryes.
	
	result[0] = True is a mesh with the hmAttr, false is not
	result[1] = currentType is the type of hm in mesh: in out or multi
	result[2] = currentAttribute is the attribute found
	'''
	origMeshIdentifyer = gVar.outMeshToCompound
	inMeshFromCompoundIdentifyer = gVar.inMeshFromCompound
	
	result = [False, False, False]
	
	#Get the attributes
	currentAttributes = cmds.listAttr(inputMesh, ud=True)
	# print '\n\nattributi trovati su %s: %s' % (inputMesh, currentAttributes)
	#Count the times that we find an attribute
	count = 0
	
	if currentAttributes != None:
		# print 'esiste un attributo custom'
		for attribute in currentAttributes:
			# print 'verifico', attribute
			#Result mesh chase
			if attribute == inMeshFromCompoundIdentifyer:
				# print 'found hm attiibute of type in'
				currentAttribute = inMeshFromCompoundIdentifyer
				currentType = 'in'
				# print 'Result Mesh'
				count += 1
				
			#Original mesh chase
			if attribute == origMeshIdentifyer:
				# print 'found hm attiibute of type out'
				currentAttribute = origMeshIdentifyer
				currentType = 'out'
				# print 'Orig Mesh'
				count += 1
				
				
			# if attribute != origMeshIdentifyer or attribute != inMeshFromCompoundIdentifyer:
				# print 'no hm custom attr.. continue...'
				# #If the custom attribute is nothing interesting, return false
				# result = [False, False, False]
				
	
		#When the count is greater than 1, we will end up with a 'multi type'
		if count > 1:
			result[0] = True
			result[1] = 'multi'
			result[2] = inMeshFromCompoundIdentifyer
		
		if count == 1:
			result[0] = True
			result[1] = currentType
			result[2] = currentAttribute
	else:
		result = [False, False, False]
	
	# print 'end result: ', result 
	
	return result
						
def optionVarRemove(inputOptionsList):
	'''
	Remove the provided optionVars passed as a list of strings and values
	[ [name, value], [name, value],...]
	'''		
	for currentOption in inputOptionsList:
		cmds.optionVar( remove= currentOption[0] )


def optionVarInit(inputOptionsList):
	'''
	This function checks if the current option var exist.
	The option var type is taken from the python type()

	int = intValue
	float = floatValue
	list = [floatValue, floatValue, floatValue]
	'''

	#For every option we check if an option var is defined, if yes we sobstitute the value with the saved one
	# if not we create the optionVar with the default value

	for currentOption in inputOptionsList:

		### REMOVE #####
		# cmds.optionVar( remove= currentOption[0] )
		# print 'REMOVE: ', currentOption
		### REMOVE #####


		# print 'query: ', currentOption
		#If option var exist, get the value and sobstitute to current
		exist = cmds.optionVar( exists= currentOption[0] )
		
		# print 'exist: ', exist
		if exist:
			currentOption[1] = cmds.optionVar( q= currentOption[0] )
			# print 'default value sobstituted with saved one', currentOption[1], currentOption[0]
		#If does not exits, create the optionVar of the specified type
		else:
			optionVarSave(inputOptionsList)

			# print 'default value created', inputOptionsList


def optionVarSave(inputOptionsList):
	'''
	This function saves the provided option var, guessing the type from the python type()

	int = intValue
	float = floatValue
	list = [floatValue, floatValue, floatValue]

	'''

	for currentOption in inputOptionsList:
		# print('saving..', currentOption, 'type', type( currentOption[1] ))
		if isinstance( currentOption[1], int):
			cmds.optionVar( intValue = [currentOption[0], currentOption[1]] )
		elif isinstance( currentOption[1], float):
			cmds.optionVar(floatValue = [currentOption[0], currentOption[1]] )
		elif isinstance( currentOption[1], str):
			cmds.optionVar( stringValue = [currentOption[0], currentOption[1]] )
		elif isinstance( currentOption[1], list):
			#If is list, check that is not the gradientControlNoAttr string list
			if isinstance(currentOption[1][0], str):
				# Clean it
				cmds.optionVar(remove = currentOption[0] )

				for element in currentOption[1]:
					cmds.optionVar(stringValueAppend =  [currentOption[0], element] )
					# print 'adding unicode!!'
			else:
				cmds.optionVar( floatValue = [currentOption[0], currentOption[1][0]] )
				cmds.optionVar( floatValueAppend = [currentOption[0], currentOption[1][1] ] )
				cmds.optionVar( floatValueAppend = [currentOption[0], currentOption[1][2] ] )

def clamp(n, minn, maxn):
    return max(min(maxn, n), minn)

	
def flatten(x):
    '''
	flatten(sequence) -> list

    Returns a single, flat list which contains all elements retrieved
    from the sequence and all recursively contained sub-sequences
    (iterables).

    Examples:
    >>> [1, 2, [3,4], (5,6)]
    [1, 2, [3, 4], (5, 6)]
    >>> flatten([[[1,2,3], (42,None)], [4,5], [6], 7, MyVector(8,9,10)])
    [1, 2, 3, 42, None, 4, 5, 6, 7, 8, 9, 10]
	'''
    result = []
    for el in x:
        #if isinstance(el, (list, tuple)):
        if hasattr(el, "__iter__") and not isinstance(el, basestring):
            result.extend(flatten(el))
        else:
            result.append(el)
    return result
