import json
import os
import hmScripts as root

root_path = root.__path__[0]

class Language(object):
    SUPPORTED_LANGUAGES = ['english', 'chinese']
    CURRENT_LANGUAGE = SUPPORTED_LANGUAGES[0]

    def __init__(self):
        # Check the language to load and set it
        #
        try:
            with open(os.path.join(root_path, 'language.txt')) as l_file:
                lang = l_file.readlines()[0].strip()
                self.set(lang)
        except IOError:
            pass

        self._values = Language.read()

    @staticmethod
    def set(lang):
        if lang in Language.SUPPORTED_LANGUAGES:
            Language.CURRENT_LANGUAGE = lang
            print('HM language set to {}'.format(lang))
        else:
            print('HM language {} not supported'.format(lang))

    @staticmethod
    def get():
        return Language.CURRENT_LANGUAGE

    @property
    def values(self):
        return self._values

    @staticmethod
    def read():
        """Read the language json file and return the dictionary"""

        filepath = os.path.join(root_path, 'UI', 'lang', Language.CURRENT_LANGUAGE)

        with open('{}.json'.format(filepath)) as data_file:
            j_data = json.load(data_file)

        return j_data

# -------------------------------------------------------------------------------------------------------------------- #
# Turn this on or off for displaying the developing UI and activating experimental stuff...
develop = 0
intersectCommand = 0
globalScaleFactor = 0 # this activate and override a global tolerance for every hmCompound/hmOffset

# This class instance store the language file to load for every UI element
#
language = Language()

# This display only the current release UI elements by hiding the features not yet ready
RELEASE = 2.35  # 2.5 # 3
RLM_LICENSING = True
PLATFORMS = 'win', 'osx', 'linux'
PUX_LICENSE_NAME = 'hmTools.lic'
RLM_LICENSE_NAME = 'hardmesh.lic'

# If on you can not create new networks if:
# 	- one input mesh, of the hm_in_mesh type, is already involved into another operation
# 	- more than one input mesh, of the hm_out_mesh type, are chosen for a new operation
# This will avoid the branching of networks that can gives errors in the methods and probably confusion while working
AVOID_BRANCHING = 1

# The multimesh feature turns on the multiple mesh input for every new operation
MULTIMESH = 1

# -------------------------------------------------- COLOR CHANGES --------------------------------------------------- #
activeCurvesColor = 13

# -------------------------------------------------- NODES NAMES ----------------------------------------------------- #
polySmoothName = 'hmPolySmooth'
intermediateMeshName = 'hmIntermediateMesh'
hm_result_mesh = 'hm_result_mesh'
intersection_node_name = 'hmIntersector'
poly_smooth_name = 'hmPolySmooth'
intermediate_mesh_name = 'hmIntermediateMesh'
intersection_curves_name = 'hmIntersectionCurve'
offset_curves_name = 'hmFeedbackCurve'

default_shader = 'lambert1'
hm_dark_shader = 'hmDarkShader'
hm_light_shader = 'hmLightShader'
custom_shader = 'custom'
shader_from_A = 'A_shader'
shader_from_B = 'B_shader'

reference_copy = 'hmRef_of'

# ------------------------------------------------------ SHADERS ----------------------------------------------------- #
shaderOptionVarName = 'hmShaderOptionVar'
darkPhong = 'hmDarkPhong'
reflectionShader = 'hmReflection'
zebraShader = 'hmZebra'  # This can be very nice and easy to implement


# ------------------------------------------------- CONNECTIONS QUERY ------------------------------------------------ #
# These attributes are added to meshes and compound node, to keep track of
# the connections, also if there are many more nodes in between

# outMeshToCompound >>>> inMeshForCompoundA or B
#Orig mesh attr
#This will be added to the each original mesh to keep track of the connections with the compound
outMeshToCompound = 'hmMeshToCompound'

#Compound Attr #These two are added to the compound to keep track of the direct connection with the meshes
inMeshForCompoundA = 'hmMeshConnectionA'
inMeshForCompoundB = 'hmMeshConnectionB'


# outCompoundToMesh >>>> inMeshFromCompound
# This will keep track from compound to mesh
outCompoundToMesh = 'hmCompoundOutToMesh' #This will be added to compound for outputting mesh

# This will always be connected to a compund as input (outCompoundToMesh)
inMeshFromCompound = 'hmMeshFromCompound' #This will be added to the result meshes


previewAndNextCompound = 'hmCompoundToCompound'

#========= Conversion utility
#Filling strategy aliases  {'Standard':1, 'Equidistant':2, 'Experimental':3}
nodeFillStrategy = {7:1, 4:2, 10:3 }
uiFillStrategy = {1:7, 2:4, 3:10}

# --------------------------------------------- COMPOUND ATTRIBUTES NAMES -------------------------------------------- #
compound_operation = 'operation' #This invert the normals based on the performed operations
# operation = 'operation' #This invert the normals based on the performed operations

geometry_bias = 'geometryBias'
corner_bias = 'cornerBias'
section_spans = 'sectionSpans'


#Edge refinment
edge_subd = 'refineMode'
edge_subd_0 = 'Off'
edge_subd_1 = 'Chord Dist'
edge_subd_2 = 'Strict'

chord_dist = 'tolerance'
# tolerance = 'tolerance'


interpolate_blend = 'smoothFiller'

weight_A = 'weightA'
weight_B = 'weightB'

use_ramp = 'activeRampedFiller'
ramp_offset = 'rampOffset'
ramp_scale = 'rampScale'
ramp_mode = 'rampMode'  # Don't know if I'll use it
ramp_chord_dist = 'rampChordDist'  # The precision of the ramp tasselation

# active_ramped_filler = 'activeRampedFiller'
# ramp_offset = 'rampOffset'
# ramp_scale = 'rampScale'
# ramp_mode = 'rampMode' #Don't know if I'll use it
# ramped_filler = 'rampedFiller' #The array profile curve

# This is the array attribute
ramp_filler = 'rampedFiller'
# ramp_base_id = 'rampedFiller'

# These are the attributes for every point
ramp_pos = 'rampedFiller_Position'
ramp_val = 'rampedFiller_FloatValue'
ramp_int = 'rampedFiller_Interp'

fill_strategy = 'fillStrategy'
smooth_filler = 'smoothFiller'
auto_merge_result = 'autoMergeResult'
merge_dist = 'mergeDist'
compute_normals = 'computeNormals'

# ---------------------------------------------- OFFSET ATTRIBUTES NAMES --------------------------------------------- #
offset_A = 'offsetA'
offset_B = 'offsetB'
boolean_operation = 'booleanOp'

#Options
remove_curl = 'removeCurl'
scale_factor = 'scaleFactor'

# --------------------------------------------- GENERIC ATTRIBUTES NAMES --------------------------------------------- #
in_meshes = 'inMeshes'
in_meshes_matrix = 'inMeshesMatrix'
out_meshes = 'outMeshes'
out_meshes_matrix = 'outMeshesMatrix'
in_curves = 'inCurves'
in_curves_matrix = 'inCurvesMatrix'
out_curves = 'outCurves'
out_curves_matrix = 'outCurvesMatrix'

bindBias = 'bindingBias'
curveSequencer = 'curveSequencer'
sequenceBias = 'sequenceBias'

# ----------------------------------------------------- TAGS ATTR ---------------------------------------------------- #
# These attributes are added on every transform that we will handle in the workflow: input meshes,
# output meshes and curves. We use them to identify the right kind of object

hm_in_mesh = 'hmInMesh'
hm_out_mesh = 'hmOutMesh'
hm_inter_curve = 'hmInterCurve'

# -------------------------------------------------- NETWORK NAMES --------------------------------------------------- #
hm_network_grp = 'hmNetwork'
hm_intersectors_grp = 'hmIntersectors'  # transform for intersector nodes
hm_curves_grp = 'hmCurves'  # Stores the output of intersectors


# #========================================= OPERATION TYPES ====================================================
# # These are the operations type allowed
#
# operationA = 'Union'
# operationB = 'Difference A-B'
# operationC = 'Difference B-A'
# operationD = 'Intersection'
# operationE = 'Panel A-B'
# operationF = 'Panel B-A'


# ---------------------------------------------- CUSTOM ATTRIBUTES NAMES --------------------------------------------- #
# Scene added attributes
operation_type = 'opType'
offset_attr_name = 'hmOffset'
smooth_attr_name = 'hmDivisions'
sub_ref_attr_name = 'refMeshVis'

# Network attributes added for handling visibility, status and computing
# Connected to the compute nodes of the network
operator_compute_name = 'operator_compute'
# This will keep track of the bypass status, on or off
operation_status_name = 'op_status'

# These three are connected to the override visibility
base_meshes_vis_name = 'base_meshes_visibility'
result_mesh_vis_name = 'result_mesh_visibility'
curve_vis_name = 'curves_visibility'

# This is the input objects shading status
shading_status = 'base_meshes_shading'


